@@ -1,7 +1,9 @@
 import os
+import threading
 import urllib.parse
+from typing import Optional
 
-from platypush.context import get_bus, get_plugin
+from platypush.context import get_bus
 from platypush.plugins.media import PlayerState, MediaPlugin
 from platypush.message.event.media import MediaPlayEvent, MediaPlayRequestEvent, \
     MediaPauseEvent, MediaStopEvent, MediaSeekEvent, MediaVolumeChangedEvent, \
@@ -47,6 +49,9 @@ class MediaVlcPlugin(MediaPlugin):
         self._on_stop_callbacks = []
         self._title = None
         self._filename = None
+        self._monitor_thread: Optional[threading.Thread] = None
+        self._on_stop_event = threading.Event()
+        self._stop_lock = threading.RLock()
 
     @classmethod
     def _watched_event_types(cls):
@@ -64,11 +69,18 @@ class MediaVlcPlugin(MediaPlugin):
 
     def _init_vlc(self, resource):
         import vlc
+
+        if self._instance:
+            self.logger.info('Another instance is running, waiting for it to terminate')
+            self._on_stop_event.wait()
+
         self._reset_state()
 
         for k, v in self._env.items():
             os.environ[k] = v
 
+        self._monitor_thread = threading.Thread(target=self._player_monitor)
+        self._monitor_thread.start()
         self._instance = vlc.Instance(*self._args)
         self._player = self._instance.media_player_new(resource)
 
@@ -76,18 +88,27 @@ class MediaVlcPlugin(MediaPlugin):
             self._player.event_manager().event_attach(
                 eventtype=evt, callback=self._event_callback())
 
+    def _player_monitor(self):
+        self._on_stop_event.wait()
+        self.logger.info('VLC stream terminated')
+        self._reset_state()
+
     def _reset_state(self):
-        self._latest_seek = None
-        self._title = None
-        self._filename = None
+        with self._stop_lock:
+            self._latest_seek = None
+            self._title = None
+            self._filename = None
+            self._on_stop_event.clear()
 
-        if self._player:
-            self._player.release()
-            self._player = None
+            if self._player:
+                self.logger.info('Releasing VLC player resource')
+                self._player.release()
+                self._player = None
 
-        if self._instance:
-            self._instance.release()
-            self._instance = None
+            if self._instance:
+                self.logger.info('Releasing VLC instance resource')
+                self._instance.release()
+                self._instance = None
 
     @staticmethod
     def _post_event(evt_type, **evt):
@@ -105,7 +126,7 @@ class MediaVlcPlugin(MediaPlugin):
                 self._post_event(MediaPauseEvent)
             elif event.type == EventType.MediaPlayerStopped or \
                     event.type == EventType.MediaPlayerEndReached:
-                self._reset_state()
+                self._on_stop_event.set()
                 self._post_event(MediaStopEvent)
                 for cbk in self._on_stop_callbacks:
                     cbk()
@@ -135,11 +156,11 @@ class MediaVlcPlugin(MediaPlugin):
         return callback
 
     @action
-    def play(self, resource, subtitles=None, fullscreen=None, volume=None):
+    def play(self, resource=None, subtitles=None, fullscreen=None, volume=None):
         """
         Play a resource.
 
-        :param resource: Resource to play - can be a local file or a remote URL
+        :param resource: Resource to play - can be a local file or a remote URL (default: None == toggle play).
         :type resource: str
 
         :param subtitles: Path to optional subtitle file
@@ -154,7 +175,10 @@ class MediaVlcPlugin(MediaPlugin):
         :type fullscreen: bool
         """
 
-        get_bus().post(MediaPlayRequestEvent(resource=resource))
+        if not resource:
+            return self.pause()
+
+        self._post_event(MediaPlayRequestEvent, resource=resource)
         resource = self._get_resource(resource)
 
         if resource.startswith('file://'):
@@ -210,14 +234,14 @@ class MediaVlcPlugin(MediaPlugin):
         """ Volume down by (default: 10)% """
         if not self._player:
             return None, 'No vlc instance is running'
-        return self.set_volume(self._player.audio_get_volume()-step)
+        return self.set_volume(int(max(0, self._player.audio_get_volume()-step)))
 
     @action
     def volup(self, step=10.0):
         """ Volume up by (default: 10)% """
         if not self._player:
             return None, 'No vlc instance is running'
-        return self.set_volume(self._player.audio_get_volume()+step)
+        return self.set_volume(int(min(100, self._player.audio_get_volume()+step)))
 
     @action
     def set_volume(self, volume):
